@@ -318,7 +318,8 @@ def get_response(operation, http_response):
         return (http_response, streaming_response.get_value())
     body = http_response.text.encode(encoding.lower())
     logger.debug(body)
-    if content_type in ('application/x-amz-json-1.1', 'application/json'):
+    if content_type in ('application/x-amz-json-1.0',
+                        'application/x-amz-json-1.1', 'application/json'):
         json_response = JSONResponse(operation)
         json_response.parse(body)
         return (http_response, json_response.get_value())
